#include "FNFrame.h"
#include "EditWindow.h"
#include "FaNegar.h"
#ifdef WIN32
    #include "OptsDialog.h"
    PegRect kbRect;
    PegRect evRect;
#else
    #include "CPKeyboardManager.h"
#endif

StorageManager *fileDlg;
CPString Dir, Name;

bool Recur;
bool IsClosing;
bool FileDlgExecuted;

extern PegBitmap gbEditNoBitmap;
extern PegBitmap gbEditYesBitmap;
struct CPMultiButtonDescription Modes[] = {
    { &gbEditNoBitmap, ID_EDITNO },
    { &gbEditYesBitmap, ID_EDITYES },
    { NULL, NULL },
};

PegMenuDescriptionML EditFileMenu[] = {
    { "Edit Mode", CMN_NO_ID, ID_EDIT, AF_ENABLED|BF_CHECKABLE|BF_CHECKED, NULL },
    DECLARE_MENU_LINE
	{ NULL, PEDIT_POPUP_SAVEASTT, ID_SAVEAS, AF_ENABLED, NULL },
   	{ NULL, GEO_POPUP_SAVE, ID_SAVE, AF_ENABLED, NULL },
	{ NULL, CMN_OPEN, ID_OPEN, AF_ENABLED, NULL },
	{ NULL, CMN_NEW, ID_NEW, AF_ENABLED, NULL },
	{ NULL, CMN_NO_ID, 0, 0, NULL }
};
PegMenuDescriptionML EditEditMenu[] = {
    { NULL, CMN_MENU_ED_SELALL, FWM_SELECT_ALL, AF_ENABLED, NULL },
    DECLARE_MENU_LINE
    { NULL, CMN_DELETE, ID_DEL, AF_ENABLED, NULL },
    { NULL, CMN_MENU_ED_PASTE, PM_PASTE, AF_ENABLED, NULL },
    { NULL, CMN_MENU_ED_COPY, PM_COPY, AF_ENABLED, NULL },
    { NULL, CMN_MENU_ED_CUT, PM_CUT, AF_ENABLED, NULL },
    DECLARE_MENU_LINE
    { NULL, CMN_MENU_UNDOREDO, ID_UNDOREDO, AF_ENABLED, NULL },
	{ NULL, CMN_NO_ID, 0, 0, NULL }
};
PegMenuDescriptionML EditFormatMenu[] = {
    { "Arrange Lines", CMN_NO_ID, ID_ARRANGE_LINES, AF_ENABLED, NULL },
    { NULL, CMN_NO_ID, 0, 0, NULL}
};
PegMenuDescriptionML EditPageMenu[] = {
    { "Go to...", CMN_NO_ID, ID_GOTO, AF_ENABLED, NULL },
    DECLARE_MENU_LINE
    { NULL, EACT_STORAGE_NEXT, ID_NEXT, AF_ENABLED, NULL},
    { NULL, GRAPH_ZOOM_PRE, ID_PREV, AF_ENABLED, NULL},
    DECLARE_MENU_LINE
    { NULL, CMN_DELETE, ID_DELPAGE, AF_ENABLED, NULL},
    { NULL, GRAPH_MENU_TEINS, ID_INSPAGE, AF_ENABLED, NULL},
	{ NULL, CMN_NO_ID, 0, 0, NULL }
};
#ifdef WIN32
PegMenuDescriptionML EditPCMenu[] = {
    { "Conv & Paste", CMN_NO_ID, ID_CONV_PASTE, AF_ENABLED, NULL },
    { "Conv & Copy", CMN_NO_ID, ID_CONV_COPY, AF_ENABLED, NULL },
    DECLARE_MENU_LINE
    { "Options...", CMN_NO_ID, ID_OPTS, AF_ENABLED, NULL },
    DECLARE_MENU_LINE
    { "Farsi Keys", CMN_NO_ID, ID_FKEYB, AF_ENABLED|BF_CHECKABLE|BF_CHECKED, NULL },
    { NULL, CMN_NO_ID, 0, 0, NULL }
};
#endif
PegMenuDescriptionML EditMainMenu[] = {
    #ifdef WIN32
        { "PC", CMN_NO_ID, 0, AF_ENABLED, EditPCMenu },
    #endif
	{ NULL, PRE_STATUS_PAGE, 0, AF_ENABLED, EditPageMenu },
	{ "Format", CMN_NO_ID, 0, AF_ENABLED, EditFormatMenu },
	{ NULL, CMN_MENU_EDIT, ID_EDITMENU, AF_ENABLED, EditEditMenu},
	{ NULL, CMN_MENU_FILE, 0, AF_ENABLED, EditFileMenu },
	{ NULL, CMN_NO_ID, 0, 0, NULL }
};

void EditWindow::Draw()
{
	BeginDraw();
	Invalidate(mReal);
	DrawFrame();
	#ifdef WIN32
    	if (mReal != GetMainFrame()->FullAppRectangle())
        {
            ed->Resize(mReal);
            vw->Resize(mReal);
        }
        else
        {
            if (vState == true)
            {
                fKeypad->Resize(kbRect);
                ed->Resize(evRect);
                vw->Resize(evRect);
            }
            else
            {
                ed->Resize(mReal);
                vw->Resize(mReal);
            }
        }
    #else
        ed->Resize(mReal);
        vw->Resize(mReal);
    #endif
	DrawChildren();
    EndDraw();
}

void EditWindow::AddUI()
{
    CPTabbedWindow *tui = (CPTabbedWindow*)m_ui;
    CPWindow *p1 = tui->GetFirstPane();
    CPWindow *p2 = tui->GetSecondPane();
    
    PegRect rr = GetToolbarButtonRect();
    PegRect rh = {0, 0, 12, 14};
    
    PegBitmapButton *b;
    PegTextButton *t;
    PegPrompt *p;
    
    // Pane 1
    extern PegBitmap gbNewBitmap;
    b = new PegBitmapButton(rr, &gbNewBitmap, ID_NEW);
    p1->AddToolbarButton(b);
    
    extern PegBitmap gbOpenBitmap;
    b = new PegBitmapButton(rr, &gbOpenBitmap, ID_OPEN);
    p1->AddToolbarButton(b);

    extern PegBitmap gbSaveBitmap;
    b = new PegBitmapButton(rr, &gbSaveBitmap, ID_SAVE);
    p1->AddToolbarButton(b);
    
    p = new PegPrompt(10, 10, 6, " ");
    p1->AddToolbarButton(p);

    evState = new CPMultiButton(rr, Modes);
    p1->AddToolbarButton(evState);
    
    p1->AddToolbarButton(p);
    
    t = new PegTextButton(rh, "<", ID_PREV);
    t->SetFont(PegTextThing::GetBoldFont());
    p1->AddToolbarButton(t);
    
    extern PegBitmap gbGotoBitmap;
    b = new PegBitmapButton(rr, &gbGotoBitmap, ID_GOTO);
    p1->AddToolbarButton(b);
    
    t = new PegTextButton(rh, ">", ID_NEXT);
    t->SetFont(PegTextThing::GetBoldFont());
    p1->AddToolbarButton(t);
    
    // Pane 2
    extern PegBitmap gbUndoRedoBitmap;
    b = new PegBitmapButton(rr, &gbUndoRedoBitmap, ID_UNDOREDO);
    p2->AddToolbarButton(b);
    
    p = new PegPrompt(10, 10, 6, " ");
    p2->AddToolbarButton(p);

    extern PegBitmap gbCutBitmap;
    b = new PegBitmapButton(rr, &gbCutBitmap, PM_CUT);
    p2->AddToolbarButton(b);

    extern PegBitmap gbCopyBitmap;
    b = new PegBitmapButton(rr, &gbCopyBitmap, PM_COPY);
    p2->AddToolbarButton(b);

    extern PegBitmap gbPasteBitmap;
    b = new PegBitmapButton(rr, &gbPasteBitmap, PM_PASTE);
    p2->AddToolbarButton(b);
    
    p2->AddToolbarButton(p);
    
    t = new PegTextButton(rh, "<", ID_PREV);
    t->SetFont(PegTextThing::GetBoldFont());
    p2->AddToolbarButton(t);
    
    t = new PegTextButton(rh, ">", ID_NEXT);
    t->SetFont(PegTextThing::GetBoldFont());
    p2->AddToolbarButton(t);
}

PegMenuDescriptionML* EditWindow::GetMenuDescriptionML()
{
	return EditMainMenu;
}

void EditWindow::Init()
{
    ed = new FaTextBox(mReal);
    Add(ed);
    
    vw = new PegTextBox(mReal, 0, AF_ENABLED|FF_RECESSED|TJ_RIGHT|TT_COPY, "", 65535);
    vw->SetScrollMode(WSM_AUTOSCROLL);
    extern PegFont FaClassPad;
    vw->SetFont(&FaClassPad);
    
    Dir = "main";
	Name = "";
	Saved = false;
	ed->DataSet("");
	ed->RemoveUndo("");
	
	sText = "";
	
    PageNum = 1;
    PageCount = 1;
    
    #ifdef WIN32
        kbRect = mReal;
        kbRect.wTop = kbRect.wBottom - fKeypad->kHeight();
        fKeypad = new FaKeypad(kbRect);
        fKeypad->RemoveStatus(PSF_ACCEPTS_FOCUS);
        
        evRect = mReal;
        evRect.wBottom = kbRect.wTop;
        ed->Resize(evRect);
        vw->Resize(evRect);
        
        Add(fKeypad);
        vState = true;
    #else
        ed->Resize(mReal);
        vw->Resize(mReal);
    #endif
    
    FileDlgExecuted = false;
    IsClosing = false;
    
    Presentation()->Redraw();
}

/** Displays the file name and the page number in the status bar. */
void EditWindow::SetCorrectStat()
{
    PegStatusBar* stbar = GetStatusBar();
    if (Saved)
    {
        CPString fName(Dir);
        fName += "\\";
	    fName += Name;
	    fName += " (";
        CP_CHAR buff[10];
        fName += (PEGCHAR*)CP_IntToString(PageNum, buff);
        fName += "/";
        fName += (PEGCHAR*)CP_IntToString(PageCount, buff);
        fName += ")";
	    stbar->SetTextField(1, fName);
    }
    else
    {
        stbar->SetTextField(1, "Untitled (1/1)");
    }
}

/** Retutns a bool that determines whether a dialog box is running or not. */
bool EditWindow::Dlg()
{
    return ((FNFrame*)GetMainFrame())->DialogRunning;
}

/** Functions that deal with dialog boxes, set this function to 'true' when a
dialog is running and set it to 'false' when no dialog is running. */
void EditWindow::Dlg(bool dState)
{
    ((FNFrame*)GetMainFrame())->DialogRunning = dState;
}

/** Displays a message box and returns the ID of its clicked button, but it
doesn't return 'IDB_CLOSE'. Instead, it returns IDB_CANCEL or IDB_NO. */
WORD EditWindow::MsgBox(PEGCHAR *title, PEGCHAR *text, bool haveCancel)
{
    if (mReal == GetMainFrame()->BottomAppRectangle())
    {
        GetMainFrame()->SwapWindows();
        Presentation()->Redraw();
    }
    
    CPString Comment(text);
    Comment += "\n\nFile: ";
    if (Saved)
    {
        Comment += Dir;
        Comment += "\\";
        Comment += Name;
    }
    else
    {
        Comment += "Untitled";
    }
    
    WORD msgStyle = MW_YES|MW_NO;
    if (haveCancel) msgStyle |= MW_CANCEL;
    PegMessageWindow *mwin = new PegMessageWindow(title, "", Comment, msgStyle, 0, NULL, 0);
    
    Dlg(true);
    WORD ans = mwin->Execute();
    Dlg(false);
    
    if (ans == IDB_CLOSE)
        return (haveCancel ? IDB_CANCEL : IDB_NO);
    else
        return ans;
}

/** Makes the environment ready for editing a new file. */
void EditWindow::New()
{
    Dir = "main";
	Name = "";
	Saved = false;
	SetCorrectStat();
	vw->DataSet("");
	ed->DataSet("");
	ed->RemoveUndo("");
	
	if (sText != "") delete[] sText;
    sText = new PEGCHAR[CP_StringByteLen((CP_CHAR*)ed->DataGet())+1];
    CP_StringCopy((CP_CHAR*)sText, (CP_CHAR*)ed->DataGet());
	
	Edit();
	PageNum = 1;
	PageCount = 1;
}

/** Shows an 'Open' dialog box an opens the selected file. */
void EditWindow::Open()
{
    MessageQueue()->Flush();
    
    fileDlg = new StorageManager(&Name, &Dir, STORAGE_ACTION_OPEN, IMU_MCS_TypeMem);
    
    Dlg(true);
	int ans = fileDlg->Execute();
	Dlg(false);
    if (ans != IDB_STORAGE_OPEN) 
        return;
    RealOpen();
}

/** Actually opens the file 'Dir/Name' ('Dir' and 'Name' are global vars). */
void EditWindow::RealOpen()
{
 	CPReadMCSFile f(Name.Text(), Dir.Text(), IMU_MCS_TypeMem);
	
	ed->DataSet(ReadData(f));
	if (Name == "")
	{
    	New();
    	return;
    }
    else
    {
        Saved = true;
    	SetCorrectStat();
    }
    
    ed->RemoveUndo(ed->DataGet());
    
    if (sText != "") delete[] sText;
    sText = new PEGCHAR[CP_StringByteLen((CP_CHAR*)ed->DataGet())+1];
    CP_StringCopy((CP_CHAR*)sText, (CP_CHAR*)ed->DataGet());
    
    vw->DataSet(ed->DataGet());
    Presentation()->Redraw();
}

/** Checks to see if the file is modified or not. */
bool EditWindow::IsModified()
{
    CPString str1(ed->DataGet());
    CPString str2(sText);
    return ((str1 == str2) ? false : true);
}

/** Displays the 'Save Changes?' dialog if needed, then save the file according
to the dialog's answer, and returns the result (IDB_YES, IDB_NO, IDB_CANCEL). */
WORD EditWindow::SaveDlg(PEGCHAR *Title)
{
    WORD ans;
    if (Saved)
    {
        if (IsModified() == false)
        {
            return IDB_YES;
        }
        else
        {
            ans = MsgBox(Title, GetLang(PEDIT_POPUP_SAVECHANGE));
            if (ans == IDB_YES)
                return (Save() ? IDB_YES : IDB_CANCEL);
            else
                return ans;
        }
    }
    else
    {
        return (SaveAs() ? IDB_YES : IDB_CANCEL);
    }
}

/** Saves the current file, and if the file does not exist in the memory, shows
a 'Save' dialog box. */
bool EditWindow::Save(int mode)
{
    if (Saved)
    {
     	CPWriteMCSFile f(Name.Text(), Dir.Text(), IMU_MCS_TypeMem);
        WriteData(f, mode);
        f.Realize();
        WriteData(f, mode);
        
        if (sText != "") delete[] sText;
        sText = new PEGCHAR[CP_StringByteLen((CP_CHAR*)ed->DataGet())+1];
        CP_StringCopy((CP_CHAR*)sText, (CP_CHAR*)ed->DataGet());
    }
    else
    {
        return SaveAs();
    }
    return true;
}

/** Shows a 'Save' dialog box to save the current file with a new name and dir. */
bool EditWindow::SaveAs(int mode)
{
    MessageQueue()->Flush();
    
    CPString NewName;
    CPString NewDir;
    fileDlg = new StorageManager(&NewName, &NewDir, STORAGE_ACTION_SAVE, IMU_MCS_TypeMem);
    
    Dlg(true);
    FileDlgExecuted = true;
    int ans = fileDlg->Execute();
    FileDlgExecuted = false;
	Dlg(false);
	if (ans != IDB_STORAGE_SAVE)
        return false;
	
	if (Saved)
	{
    	int ErrNum;
    	if (ClibValExistChk((CP_CHAR*)NewDir.Text(), (CP_CHAR*)NewName.Text()))
    	    ErrNum = 1000 * BMCSDeleteVariable((CP_CHAR*)NewDir.Text(), (CP_CHAR*)NewName.Text());
    	
    	ErrNum = BMCSCopyVariable((CP_CHAR*)Dir.Text(), (CP_CHAR*)Name.Text(),
          (CP_CHAR*)NewDir.Text(), (CP_CHAR*)NewName.Text());
        if (ErrNum != 0)
        {
            CPString ErrText(GetLang(ERR_COMMON));
            ErrText += " ";
            CP_CHAR buff[10];
            ErrText += (PEGCHAR*)CP_IntToString(ErrNum, buff);
            PegMessageWindow *mwin = new PegMessageWindow(GetLang(ERR_POPUP_TITLE), ErrText.GetBuffer(), MW_OK);
            mwin->Execute();
            return false;
        }
    }
    else
    {
        CPWriteMCSFile f(NewName.Text(), NewDir.Text(), IMU_MCS_TypeMem);
        WriteData(f, mode);
        f.Realize();
        WriteData(f, mode);
        if (f.IsNotError() == false) return false;
    }    
    
    if (sText != "") delete[] sText;
    sText = new PEGCHAR[CP_StringByteLen((CP_CHAR*)ed->DataGet())+1];
    CP_StringCopy((CP_CHAR*)sText, (CP_CHAR*)ed->DataGet());
    
    Saved = true;
    Dir = NewDir;
    Name = NewName;
    Save();
	SetCorrectStat();
	return true;
}

/** Writes the file to the disk. 'mode' can be one of the following:
0 = Delete Page,    1 = Normal,    2 = Insert Page */
void EditWindow::WriteData(CPWriteFile &f, int mode)
{
    CPMEMFileHeader header("Farsi Negar", "Text");
    
	if(f.IsNotError())
	{
	    CPString text;
	    int i;
        
        if (PageCount == 1)
        {
            if (mode == 1)
            {
                header.Write(f);
                f.WriteInt(11);
                f.WriteInt(PageCount);
                text = ed->DataGet();
                text.Write(f);
                return;
            }
        }
        
        CPWriteMCSFile fo("FaTmp", f.GetPath(), IMU_MCS_TypeMem);
	    fo.SetName("FaTmp");
        fo.SetPath(f.GetPath());
	    for (i = 0; i < 100; i++)
	    {
            if (fo.FileExists())
            {
                CP_CHAR buff[4];
	            CPString fName("FaTmp");
                fName += (PEGCHAR*)CP_IntToString(i, buff);
                fo.SetName(fName.Text());
            }
            else
            {
                break;
            }
        }
        
        int ErrNum = BMCSCopyVariable((CP_CHAR*)f.GetPath(), (CP_CHAR*)f.GetName(),
          (CP_CHAR*)fo.GetPath(), (CP_CHAR*)fo.GetName());
        if (ErrNum != 0)
        {
            CPString ErrText(GetLang(ERR_COMMON));
            ErrText += " ";
            CP_CHAR buff[10];
            ErrText += (PEGCHAR*)CP_IntToString(ErrNum, buff);
            PegMessageWindow *mwin = new PegMessageWindow(GetLang(ERR_POPUP_TITLE), ErrText.GetBuffer(), MW_OK);
            mwin->Execute();
            return;
        }
        
        f.Close();
        f.Open(1);
        header.Write(f);
        f.WriteInt(11);
        f.WriteInt(PageCount);
        
        CPReadMCSFile fi(fo.GetName(), fo.GetPath(), IMU_MCS_TypeMem);
        header.Read(fi);
        fi.ReadInt();
        fi.ReadInt();
        if (mode == 0)
        {
            if (PageNum == 1)
            {
                text.Read(fi);
                for (i = -1; i < PageCount; i++)
                {
                    text.Read(fi);
                    text.Write(f);
                }
            }
            else if (PageNum == PageCount + 1)
            {
                for (i = 0; i < PageCount; i++)
                {
                    text.Read(fi);
                    text.Write(f);
                }
                PageNum--;
            }
            else
            {
                for (i = 1; i < PageNum; i++)
                {
                    text.Read(fi);
                    text.Write(f);
                }
                text.Read(fi);
                for (i = PageNum-2; i < PageCount; i++)
                {
                    text.Read(fi);
                    text.Write(f);
                }
            }
        }
        else if (mode == 1)
        {
            if (PageNum == 1)
            {
                text.Read(fi);
                text = ed->DataGet();
                text.Write(f);
                for (i = 1; i < PageCount; i++)
                {
                    text.Read(fi);
                    text.Write(f);
                }
            }
            else if (PageNum == PageCount)
            {
                for (i = 1; i < PageCount; i++)
                {
                    text.Read(fi);
                    text.Write(f);
                }
                text = ed->DataGet();
                text.Write(f);
            }
            else
            {
                for (i = 1; i < PageNum; i++)
                {
                    text.Read(fi);
                    text.Write(f);
                }
                text.Read(fi);
                text = ed->DataGet();
                text.Write(f);
                for (i = PageNum; i < PageCount; i++)
                {
                    text.Read(fi);
                    text.Write(f);
                }
            }
        }
        else if (mode == 2)
        {
            if (PageNum == 1)
            {
                text = ed->DataGet();
                text.Write(f);
                for (i = 1; i < PageCount; i++)
                {
                    text.Read(fi);
                    text.Write(f);
                }
            }
            else
            {
                for (i = 1; i < PageNum; i++)
                {
                    text.Read(fi);
                    text.Write(f);
                }
                text = ed->DataGet();
                text.Write(f);
                for (i = PageNum; i < PageCount; i++)
                {
                    text.Read(fi);
                    text.Write(f);
                }
            }
        }
        
        fi.Delete();
    }
    else
    {
        CPString ErrText(GetLang(ERR_COMMON));
        ErrText += " ";
        CP_CHAR buff[10];
        ErrText += (PEGCHAR*)CP_IntToString(f.GetErrorCode(), buff);
        PegMessageWindow *mwin = new PegMessageWindow(GetLang(ERR_POPUP_TITLE), ErrText.GetBuffer(), MW_OK);
        mwin->Execute();
        return;
    }
}

/** Reads the specified page of a file from disk. */
CPString EditWindow::ReadData(CPReadFile &f)
{
    CPMEMFileHeader header("Farsi Negar", "Text");
	header.Read(f);
	
	CPString text;
	if(f.IsNotError())
	{
        if (f.ReadInt() != 11)
        {
            PegMessageWindow *mwin = new PegMessageWindow(GetLang(ERR_POPUP_TITLE), GetLang(ERR_DATATYPE), MW_OK);
            mwin->Execute();
            Name = "";
            return "";
        }
        PageCount = f.ReadInt();
        for (int i = 0; i < PageNum; i++)
        {
            text = "";
            text.Read(f);
        }
    }
    else
    {
        PegMessageWindow *mwin = new PegMessageWindow(GetLang(ERR_POPUP_TITLE), GetLang(ERR_DATATYPE), MW_OK);
        mwin->Execute();
        Name = "";
        return "";
    }
    return text;
}

void EditWindow::InsPage()
{
    if (MsgBox(GetLang(GRAPH_MENU_TEINS), "Insert a new page?", false) == IDB_NO) return;
    
    if (SaveDlg(GetLang(GRAPH_MENU_TEINS)) == IDB_CANCEL) return;
    
    PageCount++;
    vw->DataSet("");
    ed->DataSet("");
    ed->RemoveUndo("");
    Edit();
    Save(2);
    RealOpen();
}

void EditWindow::DelPage()
{
    if (PageCount == 1)
    {
        PegMessageWindow *mwin = new PegMessageWindow(GetLang(ERR_POPUP_TITLE), "",
            "Cannot delete current page because it is the only page in the file.", MW_OK, 0, NULL, 0);
        mwin->Execute();
        return;
    }
    
    if (MsgBox(GetLang(CMN_DELETE), "Delete current page?", false) == IDB_NO) return;
    
    PageCount--;
    vw->DataSet("");
    ed->DataSet("");
    ed->RemoveUndo("");
    Edit();
    Save(0);
    RealOpen();
}

/** Navigates to the previous page. */
void EditWindow::PrevPage()
{
    if (PageNum == 1) return;
    if (SaveDlg(GetLang(GRAPH_ZOOM_PRE)) == IDB_CANCEL) return;
    PageNum--;
    RealOpen();
}

/** Navigates to the next page. */
void EditWindow::NextPage()
{
    if (PageNum == PageCount)
    {
        if (MsgBox(GetLang(EACT_STORAGE_NEXT), "Create a new page?", false) == IDB_NO) return;
        
        if (SaveDlg("Create") == IDB_CANCEL) return;
        
        PageNum++;
        PageCount++;
        vw->DataSet("");
        ed->DataSet("");
        ed->RemoveUndo("");
        
        if (sText != "") delete[] sText;
        sText = new PEGCHAR[CP_StringByteLen((CP_CHAR*)ed->DataGet())+1];
        CP_StringCopy((CP_CHAR*)sText, (CP_CHAR*)ed->DataGet());
        
        Edit();
        Save();
        RealOpen();
    }
    else
    {
        if (SaveDlg(GetLang(EACT_STORAGE_NEXT)) == IDB_CANCEL) return;
        PageNum++;
        RealOpen();
    }
}

/** Navigates to the page that user chooses. */
void EditWindow::Goto()
{
    OBCD OpgNum;
    InputDialog *pgAsk = new InputDialog("Go to...", GetLang(SETUP_PRE_CHECK_PN), OpgNum);
    PegMessageWindow *mwin;
	
    Dlg(true);
    int ans = pgAsk->Execute();
    if (ans == IDB_OK)
    {
        short pgNum;
        int error = Cal_toshort_OBC(&OpgNum, &pgNum);
        if (error == 0)
        {
            if ((pgNum < 0) || (pgNum > PageCount))
            {
                mwin = new PegMessageWindow(GetLang(ERR_POPUP_TITLE), "", GetLang(ERR_INPUT),
                  MW_OK, 0, NULL, 0);
                mwin->Execute();
            }
            else if (pgNum == 0)
            {
                if (SaveDlg("Go to...") != IDB_CANCEL)
                {
                    PageNum = PageCount;
                    RealOpen();
                }
            }
            else
            {
                if (SaveDlg("Go to...") != IDB_CANCEL)
                {
                    PageNum = pgNum;
                    RealOpen();
                }
            }
        }
        else
        {
            mwin = new PegMessageWindow(GetLang(ERR_POPUP_TITLE), GetLang(ERR_NUMERICAL), MW_OK);
            mwin->Execute();
        }
    }
    Dlg(false);
}

/** Sets up the 'View' mode. */
void EditWindow::Read()
{
    Remove(ed);
    Add(vw);
    vw->DataSet(ed->DataGet());
    vw->SetTopLine(ed->GetTopLine());
    
    ((PegMenuButton*)m_menu->Find(ID_EDITMENU, false))->SetEnabled(false);
    ((PegMenuButton*)m_menu->Find(ID_EDIT))->SetChecked(false);
    ((CPTabbedWindow*)m_ui)->ActivateFirstPane();
    ((CPTabbedWindow*)m_ui)->DisableTab();
    evState->SetIndex(1);
    Presentation()->Redraw();
}

/** Sets up the 'Edit' mode. */
void EditWindow::Edit()
{
    Remove(vw);
    Add(ed);
    ed->DataSet(vw->DataGet());
    ed->SetTopLine(vw->GetTopLine());
    
    ((PegMenuButton*)m_menu->Find(ID_EDITMENU, false))->SetEnabled(true);
    ((PegMenuButton*)m_menu->Find(ID_EDIT))->SetChecked(true);
    ((CPTabbedWindow*)m_ui)->EnableTab();
    evState->SetIndex(0);
    Presentation()->Redraw();
}

#ifdef WIN32
/** Sets the state of the Farsi keypad. */
void EditWindow::SetFKeypad(bool kState)
{
    vState = kState;
    Presentation()->Redraw();
    if (mReal == GetMainFrame()->FullAppRectangle())
    {
        if (kState == true)
            Add(fKeypad);
        else
            Remove(fKeypad);
    }
}
#endif

SIGNED EditWindow::Message(const PegMessage &Mesg)
{
    switch(Mesg.wType)
    {
        #ifndef WIN32
        case PM_FIRST_START:
            {
                CPModuleWindow::Message(Mesg);
                CPMainFrame *mf = GetMainFrame();
                mf->Keypad()->SetKeyboard(4);
                mf->ShowKeypad();
                mf->Redraw();
            }
            break;
        #endif
        case PM_CURRENT:
            CPModuleWindow::Message(Mesg);
            SetCorrectStat();
            SetDefaultFocus(ed);
            MoveFocusTree(ed);
            break;
        case PM_SIZE:
            CPModuleWindow::Message(Mesg);
            MoveFocusTree(ed);
            break;
        
        case SIGNAL(ID_NEW, PSF_CLICKED):
            {
                if (IsModified())
                {
                    WORD ans = MsgBox(GetLang(CMN_NEW), GetLang(PEDIT_POPUP_SAVECHANGE));
                    if (ans == IDB_YES)
                    {
                        if (Save() == false) break;
                    }
                    else if (ans == IDB_CANCEL)
                    {
                        break;
                    }
                }
                New();
            }
            break;
        case SIGNAL(ID_OPEN, PSF_CLICKED):
            {
                bool ReallySaved = true;
                if (IsModified())
                {
                    WORD ans = MsgBox(GetLang(CMN_OPEN), GetLang(PEDIT_POPUP_SAVECHANGE));
                    if (ans == IDB_YES)
                    {
                        ReallySaved = Save();
                    }
                    else if (ans == IDB_CANCEL)
                    {
                        break;
                    }
                }
                if (ReallySaved)
                {
                    PageNum = 1;
                    Open();
                }
            }
            break;
        case SIGNAL(ID_SAVE, PSF_CLICKED):
            Save();
            break;
        case SIGNAL(ID_SAVEAS, PSF_CLICKED):
            SaveAs();
            break;
        
        case SIGNAL(ID_UNDOREDO, PSF_CLICKED):
        case SIGNAL(PM_CUT, PSF_CLICKED):
        case SIGNAL(PM_COPY, PSF_CLICKED):
        case SIGNAL(PM_PASTE, PSF_CLICKED):
        case SIGNAL(ID_DEL, PSF_CLICKED):
        case SIGNAL(FWM_SELECT_ALL, PSF_CLICKED):
        case SIGNAL(ID_ARRANGE_LINES, PSF_CLICKED):
            ed->Message(Mesg);
            break;
        
        case SIGNAL(ID_INSPAGE, PSF_CLICKED): InsPage();  break;
        case SIGNAL(ID_DELPAGE, PSF_CLICKED): DelPage();  break;
        case SIGNAL(ID_PREV, PSF_CLICKED):    PrevPage(); break;
        case SIGNAL(ID_NEXT, PSF_CLICKED):    NextPage(); break;
        case SIGNAL(ID_GOTO, PSF_CLICKED):    Goto();     break;
        
        case SIGNAL(ID_EDIT, PSF_CHECK_OFF):
        case SIGNAL(ID_EDITYES, PSF_CLICKED):
            Recur = !(Recur);
            if (Recur) Read();
            break;
        case SIGNAL(ID_EDIT, PSF_CHECK_ON):
        case SIGNAL(ID_EDITNO, PSF_CLICKED):
            Recur = !(Recur);
            if (Recur) Edit();
            break;
        #ifdef WIN32
        case SIGNAL(ID_FKEYB, PSF_CHECK_OFF):
            SetFKeypad(false);
            break;
        case SIGNAL(ID_FKEYB, PSF_CHECK_ON):
            SetFKeypad(true);
            break;
        case FWM_KEYPAD_WAS_TOGGLED:
            if (Dlg() == false)
                SetFKeypad(vState);
            break;
        case SIGNAL(ID_OPTS, PSF_CLICKED):
            {
                Dlg(true);
                OptsDialog *optsDlg = new OptsDialog(ed);
                optsDlg->Execute();
                Dlg(false);
            }
            break;
        case SIGNAL(ID_CONV_COPY, PSF_CLICKED):
        case SIGNAL(ID_CONV_PASTE, PSF_CLICKED):
            ed->Message(Mesg);
            break;
        #endif
        
        case HM_SYS_CLOSE:
            if (IsClosing == false)
            {
                if (Dlg()) break;
                IsClosing = true;
                if (IsModified())
                {
                    WORD ans = MsgBox(GetLang(CMN_MENU_OP_CLOSE), GetLang(PEDIT_POPUP_SAVECHANGE), false);
                    if (ans == IDB_YES)
                    {
                        FileDlgExecuted = true;
                        Save();
                    }
                }
            }
            else
            {
                if (FileDlgExecuted)
                    fileDlg->Execute();
            }
            break;
        
		default:
	        return CPModuleWindow::Message(Mesg);
    }
    return 0;
}

