#include "FaTextBox.h"
#include "EditWindow.h"
#include "FaNegar.h"

void FaTextBox::Init()
{
    Style(FF_RECESSED|EF_EDIT);
    SetMaxChars(65535);
    extern PegFont FaClassPad;
    SetFont(&FaClassPad);
    SetScrollMode(WSM_AUTOSCROLL);
    urTemp = "";
    urText = "";
}

/** Performs the undo/redo. */
void FaTextBox::UndoRedo()
{
    if (urTemp != "")
        delete[] urTemp;
    urTemp = new PEGCHAR[CP_StringByteLen((CP_CHAR*)urText)+1];
    CP_StringCopy((CP_CHAR*)urTemp, (CP_CHAR*)urText);
    
    if (urText != "")
        delete[] urText;
    urText = new PEGCHAR[CP_StringByteLen((CP_CHAR*)DataGet())+1];
    CP_StringCopy((CP_CHAR*)urText, (CP_CHAR*)DataGet());
    
    DataSet(urTemp);
    mMarkAnchor = mMarkEnd;
    
    urCTemp = urCursor;
    urCursor = mCursor;
    
    mCursor = urCTemp;
    
    Redraw();
}

/** Sets an 'undo point'. */
void FaTextBox::ArrangeUndo()
{
    if (urText != "")
        delete[] urText;
    urText = new PEGCHAR[CP_StringByteLen((CP_CHAR*)DataGet()) + 1];
    CP_StringCopy((CP_CHAR*)urText, (CP_CHAR*)DataGet());
    
    urCursor = mCursor;
}

/** Clears the undo cache. Called when you want to edit a new document. If the
document has some text by default, that text should be passed to the arg
'prevUndo', otherwise an empty string should be passed to it. */
void FaTextBox::RemoveUndo(PEGCHAR* prevUndo)
{
    if (urTemp != "")
        delete[] urTemp;
    urTemp = new PEGCHAR[1];
    urTemp = "";
    
    if (urText != "")
        delete[] urText;
    urText = new PEGCHAR[CP_StringByteLen((CP_CHAR*)prevUndo)+1];
    CP_StringCopy((CP_CHAR*)urText, (CP_CHAR*)prevUndo);
    
    urCTemp.x = 0;
    urCTemp.y = 0;
    urCursor.x = 0;
    urCursor.y = 0;
}

/** Just a pointer to the cursor. */
PegPoint *FaTextBox::Cursor()
{
    return &mCursor;
}

SIGNED FaTextBox::Message(const PegMessage &Mesg)
{
    switch(Mesg.wType)
    {
        case FWM_UNDO:
        case SIGNAL(ID_UNDOREDO, PSF_CLICKED):
            UndoRedo();
            break;
        case PM_CUT:
        case SIGNAL(PM_CUT, PSF_CLICKED):
            ArrangeUndo();
            CopyToScratchPad();
            DeleteMarkedText();
            StyleLetters();
            mMarkAnchor = mMarkEnd;
            break;
        case PM_COPY:
        case SIGNAL(PM_COPY, PSF_CLICKED):
            CopyToScratchPad();
            break;
        case PM_PASTE:
        case SIGNAL(PM_PASTE, PSF_CLICKED):    
            {
                ArrangeUndo();
                PegPoint before = mCursor;
                DeleteMarkedText();
                PasteFromScratchPad();
                PegPoint after = mCursor;
                mCursor = before;
                StyleLetters(false);
                mCursor = after;
                StyleLetters();
                mMarkAnchor = mMarkEnd;
            }
            break;
        case SIGNAL(ID_DEL, PSF_CLICKED):
            {
                PegMessage keyMsg(this, PM_KEY);
                keyMsg.iData = PK_DELETE;
                MessageQueue()->Push(keyMsg);
            }
            break;
        case SIGNAL(FWM_SELECT_ALL, PSF_CLICKED):
            SelectAll();
            break;
        case SIGNAL(ID_ARRANGE_LINES, PSF_CLICKED):
            {
                ArrangeUndo();
                bool finished = false;
                CPString Text = DataGet();
                CPString tempLine, Line;
                CPString tempNextWord, nextWord;
                CPString nextLine, nextText;
                extern PegFont FaClassPad;
                int i = -1;
                while ( !finished )
                {
                    Line = "";
                    nextLine = "";
                    
                    while (1 == 1)
                    {
                        i++;
                        if (Text[i] == CP_CHAR_CR) break;
                        if (Text[i] == 0)
                        {
                            finished = true;
                            break;
                        }
                        tempLine = Line;
                        Line = Text[i];
                        Line += tempLine;
                    }
                    
                    tempLine == Line;
                    tempLine.ReplaceCharacter(' ', 0);
                    
                    if (tempLine == "")
                    {
                        nextText += CP_CHAR_CR;
                        continue;
                    }
                    
                    int j;
                    for (j = 0; j <= Line.Length(); j++)
                    {
                        tempNextWord = nextWord;
                        nextWord = Line[j];
                        nextWord += tempNextWord;
                        if ( (Line[j] == ' ') || (j == Line.Length()) )
                        {
                            tempNextWord = nextWord;
                            if (tempNextWord[0] == ' ')
                                tempNextWord = tempNextWord.Mid(1);
                            
                            if (TextWidth(nextLine, &FaClassPad) + TextWidth(tempNextWord, &FaClassPad) < 140)
                            {
                                nextLine = nextWord + nextLine;
                                nextWord = "";
                            }
                            else
                            {
                                if (nextLine[0] == ' ')
                                    nextLine = nextLine.Mid(1);
                                
                                nextText += nextLine;
                                nextText += CP_CHAR_CR;
                                
                                nextLine = nextWord;
                                nextWord = "";
                            }
                        }
                    }
                    nextText += nextLine;
                    if (!finished) nextText += CP_CHAR_CR;
                    Line = "";
                }
                DataSet(nextText);
                mMarkAnchor = mMarkEnd;
                Redraw();
            }
            break;
        #ifdef WIN32
        case SIGNAL(ID_CONV_COPY, PSF_CLICKED):
            {
                Farsi *farsi = new Farsi;
                CPString lstr;
                CPString rstr;
                CPString str;
                CPString finalstr;
                
                CopyToScratchPad();
                ((CPModuleWindow*)Parent())->GetMainFrame()->GetClipboard()->AsString(str);
                
                PEGCHAR *fstr = new PEGCHAR[CP_StringByteLen((CP_CHAR*)str.GetBuffer())+1];
                CP_StringCopy((CP_CHAR*)fstr, (CP_CHAR*)str.GetBuffer());
                
                int size = CP_StringByteLen((CP_CHAR*)fstr) + 1;
                
                str = "";
                CPMCHAR chr;
                while (size >= 0)
                {
                    chr = (CP_CHAR)(fstr[--size]);
                    if (chr == 0xEC)
                    {
                        chr = ChrToPChr(0xEC00 + (CP_CHAR)(fstr[size+1]) );
                        --size;
                    }
                    else if ((chr == 0xED) || (chr == 0xEE))
                    {
                        chr = lqmark;
                        --size;
                    }
                    else
                    {
                        chr = (CP_CHAR)(fstr[size+1]);
                    }
                    
                    if (yehTOyehd)
                    {
                        if (chr == pk_yeh)
                            chr =pk_yehd;
                    }
                    if (kafTOakaf)
                    {
                        if (chr == pk_kaf)
                            chr == pk_akaf;
                    }
                    if (CNVparen)
                    {
                        if (chr == clparen)
                            chr = olparen;
                        else if (chr == olparen)
                            chr = clparen;
                    }
                    if (CNVsbrac)
                    {
                        if (chr == csbrac)
                            chr = osbrac;
                        else if (chr == osbrac)
                            chr = csbrac;
                    }
                    if (CNVcbrac)
                    {
                        if (chr == ccbrac)
                            chr = ocbrac;
                        else if (chr == ocbrac)
                            chr = ccbrac;
                    }
                    if (CNVquote)
                    {
                        if (chr == pk_cquote)
                            chr = pk_oquote;
                        else if (chr == pk_oquote)
                            chr = pk_cquote;
                    }
                    if (CNVgtlt)
                    {
                        if (chr == lts)
                            chr = gts;
                        else if (chr == gts)
                            chr = lts;
                    }
                    
                    int i;
                    int iEnd = (chr > 0xFF) ? 2 : 1;
                    CPMCHAR realchr = chr;
                    for (i = 1; i <= iEnd; i++)
                    {
                        if (realchr > 0xFF)
                        {
                            chr = (i == 1) ? ((realchr >> 8) & 0xFF) : (realchr & 0xFF);
                        }
                        
                        if (farsi->Type( *(new FaChar(chr)) ) == NIL)
                        {
                            str = str + lstr; lstr = "";
                            str = str + rstr; rstr = "";
                            CPString tempcrlf(0xD); tempcrlf += 0xA;
                            finalstr = tempcrlf + finalstr;
                            finalstr = str + finalstr;
                            str = "";
                        }
                        else if (farsi->Type( *(new FaChar(chr)) ) == SPACE)
                        {
                            str = str + lstr; lstr = "";
                            str = str + rstr; rstr = "";
                            str = str + (PEGCHAR*)" ";
                        }
                        else if (farsi->Type( *(new FaChar(chr)) ) == RTL)
                        {
                            str = str + lstr; lstr = "";
                            lstr = "";
                            rstr += (PEGCHAR)chr;
                        }
                        else
                        {
                            str = str + rstr; rstr = "";
                            rstr = "";
                            CPString tempchar(chr);
                            lstr = tempchar + lstr;
                        }
                    }
                }
                str = str + lstr;
                str = str + rstr;
                CPString tempcrlf(0xD); tempcrlf += 0xA;
                finalstr = tempcrlf + finalstr;
                finalstr = str + finalstr; 
                finalstr = tempcrlf + finalstr;
                
                Presentation()->SetScratchPad(finalstr.Text());
                delete [] fstr;
            }
            break;
        case SIGNAL(ID_CONV_PASTE, PSF_CLICKED):
            {
                HomeCursor();
                SendKeyMsg(PK_CR);
                
                CPString str;
                ((CPModuleWindow*)Parent())->GetMainFrame()->GetClipboard()->AsString(str);
                
                PEGCHAR *fstr = new PEGCHAR[CP_StringByteLen((CP_CHAR*)str.GetBuffer())+1];
                CP_StringCopy((CP_CHAR*)fstr, (CP_CHAR*)str.GetBuffer());
                
                int size = CP_StringByteLen((CP_CHAR*)fstr);
                for (int i = 0; i < size; i++)
                {
                    InsertCharEx((CP_CHAR)(*(fstr++)));
                }
                InsertCharEx(0);
                InsertCharEx(0);
                InsertCharEx(0);
                
                mMarkAnchor = mMarkEnd;
                
                delete [] fstr;
                
                Redraw();
            }
            break;
        #endif
        case PM_KEY:
            switch(Mesg.iData)
            {
                case 1:
                    SelectAll();
                    break;
                case 2: //LDel
                    if ((mCursor.x == 0) && (mCursor.y == 0)) break;
                    ArrangeUndo();
                    if (mMarkAnchor == mMarkEnd)
                    {
                        RemoveCharAtCursor(true);
                        StyleLetters();
                    }
                    else
                    {
                        DeleteMarkedText();
                        StyleLetters();
                        mMarkAnchor = mMarkEnd;
                    }
                    break;
                case PK_DELETE: //RDel
                    ArrangeUndo();
                    if (mMarkAnchor == mMarkEnd)
                    {
                        RemoveCharAtCursor(false);
                        StyleLetters();
                    }
                    else
                    {
                        DeleteMarkedText();
                        StyleLetters();
                        mMarkAnchor = mMarkEnd;
                    }
                    break;
                case 3: //LINE
                    {
                        ArrangeUndo();
                        DeleteMarkedText();
                        PegPoint before = mCursor;
                        InsertCharAtCursor(CP_CHAR_CR);
                        PegPoint after = mCursor;
                        mCursor = before;
                        StyleLetters();
                        mCursor = after;
                        StyleLetters();
                        mMarkAnchor = mMarkEnd;
                    }
                    break;
                case 4: //ABJD
                    if (mMarkAnchor == mMarkEnd)
                    {
                        ArrangeUndo();
                        StyleLetters();
                    }
                    break;
                case PK_BACKSPACE: //Backspace
                    ArrangeUndo();
                    if (mMarkAnchor == mMarkEnd)
                    {
                        Backspace();
                    }
                    else
                    {
                        DeleteMarkedText();
                        StyleLetters();
                        mMarkAnchor = mMarkEnd;
                    }
                    break;
                case PK_TAB:
                    ArrangeUndo();
                    DeleteMarkedText();
                    StyleLetters(true, *(new FaChar(0x20)));
                    mMarkAnchor = mMarkEnd;
                    break;
                case PK_CR:
                    ArrangeUndo();
                    DeleteMarkedText();
                    NewLine();
                    mMarkAnchor = mMarkEnd;
                    Redraw();
                    break;
                case PK_LNUP: case PK_LNDN: case PK_LEFT: case PK_RIGHT:
                case PK_HOME: case PK_END:
                    CPEditBox::Message(Mesg);
                    if (!shift) mMarkAnchor = mMarkEnd;
                    break;
                case PK_SHIFT:
                    CPEditBox::Message(Mesg);
                    shift = true;
                    break;
                #ifdef WIN32
                    //Most of these keys do nothing. But if you omit handling them
                    //  here they will insert unwanted chars into the editor.
                    case PK_ESC:     case PK_INSERT:     case PK_ALT:        case PK_CTRL:
                    case PK_PGUP:       case PK_PGDN:
                    case PK_CAPS:    case PK_NUMLOCK:    case PK_SCROLLLOCK: case PK_PAUSEBREAK:
                    case PK_F1:      case PK_F2:         case PK_F3:         case PK_F4:
                    case PK_F5:      case PK_F6:         case PK_F7:         case PK_F8:
                    case PK_F9:      case PK_F10:        case PK_F11:        case PK_F12:
                        CPEditBox::Message(Mesg);
                        break;
                    case pk_line   : SendKeyMsg(4);       break;
                    case pk_peh    : SendKeyMsg(peh);     break;
                    case pk_cheh   : SendKeyMsg(cheh);    break;
                    case pk_jeh    : SendKeyMsg(jeh);     break;
                    case pk_gaf    : SendKeyMsg(gaf);     break;
                    case pk_kaf    : SendKeyMsg(kaf);     break;
                    case pk_vspace : SendKeyMsg(vspace);  break;
                    case pk_vconn  : SendKeyMsg(vconn);   break;
                    case pk_comma  : SendKeyMsg(comma);   break;
                    case pk_oquote : SendKeyMsg(oquote);  break;
                    case pk_scolon : SendKeyMsg(scolon);  break;
                    case pk_cquote : SendKeyMsg(cquote);  break;
                    case pk_qmark  : SendKeyMsg(qmark);   break;
                    case pk_hamza  : SendKeyMsg(hamza);   break;
                    case pk_aa     : SendKeyMsg(aa);      break;
                    case pk_halef  : SendKeyMsg(halef);   break;
                    case pk_hvav   : SendKeyMsg(hvav);    break;
                    case pk_alefh  : SendKeyMsg(alefh);   break;
                    case pk_hyeh   : SendKeyMsg(hyeh);    break;
                    case pk_alef   : SendKeyMsg(alef);    break;
                    case pk_beh    : SendKeyMsg(beh);     break;
                    case pk_dheh   : SendKeyMsg(dheh);    break;
                    case pk_teh    : SendKeyMsg(teh);     break;
                    case pk_ceh    : SendKeyMsg(ceh);     break;
                    case pk_jim    : SendKeyMsg(jim);     break;
                    case pk_ha     : SendKeyMsg(ha);      break;
                    case pk_kheh   : SendKeyMsg(kheh);    break;
                    case pk_dal    : SendKeyMsg(dal);     break;
                    case pk_zal    : SendKeyMsg(zal);     break;
                    case pk_reh    : SendKeyMsg(reh);     break;
                    case pk_zeh    : SendKeyMsg(zeh);     break;
                    case pk_sin    : SendKeyMsg(__sin);   break;
                    case pk_shin   : SendKeyMsg(shin);    break;
                    case pk_sad    : SendKeyMsg(sad);     break;
                    case pk_zad    : SendKeyMsg(zad);     break;
                    case pk_multis : SendKeyMsg(multis);  break;
                    case pk_divs   : SendKeyMsg(divs);    break;
                    case pk_ta     : SendKeyMsg(ta);      break;
                    case pk_za     : SendKeyMsg(za);      break;
                    case pk_ein    : SendKeyMsg(ein);     break;
                    case pk_ghein  : SendKeyMsg(ghein);   break;
                    case pk_tatweel: SendKeyMsg(tatweel); break;
                    case pk_feh    : SendKeyMsg(feh);     break;
                    case pk_ghaf   : SendKeyMsg(ghaf);    break;
                    case pk_akaf   :
                      SendKeyMsg(akafTOkaf ? kaf : akaf); break;
                    case pk_lam    : SendKeyMsg(lam);     break;
                    case pk_mim    : SendKeyMsg(mim);     break;
                    case pk_noon   : SendKeyMsg(noon);    break;
                    case pk_heh    : SendKeyMsg(heh);     break;
                    case pk_vav    : SendKeyMsg(vav);     break;
                    case pk_yeh    : SendKeyMsg(yeh);     break;
                    case pk_yehd   :
                      SendKeyMsg(yehdTOyeh ? yeh : yehd); break;
                    case pk_tfath  :                      break;
                    case pk_tzam   :                      break;
                    case pk_tjar   :                      break;
                    case pk_fath   :                      break;
                    case pk_zam    :                      break;
                    case pk_jar    :                      break;
                    case pk_tashdid:                      break;
                    case pk_sokun  :                      break;
                #endif
                default:
                    ArrangeUndo();
                    DeleteMarkedText();
                    #ifdef WIN32
                        WORD newChar = Mesg.iData;
                        if (shspaceTOvspace)
                        {
                            if ((Mesg.iData == space) && (shift)) newChar = vspace;
                        }
                        if (ldigitTOdigit)
                        {
                            switch (Mesg.iData)
                            {
                                case ldigit0: newChar = digit0; break;
                                case ldigit1: newChar = digit1; break;
                                case ldigit2: newChar = digit2; break;
                                case ldigit3: newChar = digit3; break;
                                case ldigit4: newChar = digit4; break;
                                case ldigit5: newChar = digit5; break;
                                case ldigit6: newChar = digit6; break;
                                case ldigit7: newChar = digit7; break;
                                case ldigit8: newChar = digit8; break;
                                case ldigit9: newChar = digit9; break;
                            }
                        }
                        if (lparenTOparen)
                        {
                            if (CNVparen)
                            {
                                if (Mesg.iData == clparen)
                                    newChar = oparen;
                                else if (Mesg.iData == olparen)
                                    newChar = cparen;
                            }
                            else
                            {
                                if (Mesg.iData == clparen)
                                    newChar = cparen;
                                else if (Mesg.iData == olparen)
                                    newChar = oparen;
                            }
                        }
                        else
                        {
                            if (CNVparen)
                            {
                                if (Mesg.iData == clparen)
                                    newChar = olparen;
                                else if (Mesg.iData == olparen)
                                    newChar = clparen;
                            }
                        }
                        if (CNVsbrac)
                        {
                            if (Mesg.iData == csbrac)
                                newChar = osbrac;
                            else if (Mesg.iData == osbrac)
                                newChar = csbrac;
                        }
                        if (CNVcbrac)
                        {
                            if (Mesg.iData == ccbrac)
                                newChar = ocbrac;
                            else if (Mesg.iData == ocbrac)
                                newChar = ccbrac;
                        }
                        if (CNVquote)
                        {
                            if ((CP_CHAR)(Mesg.iData)+0xEC00 == cquote)
                                newChar = oquote;
                            else if ((CP_CHAR)(Mesg.iData)+0xEC00 == oquote)
                                newChar = cquote;
                        }
                        if (CNVgtlt)
                        {
                            if (Mesg.iData == lts)
                                newChar = gts;
                            else if (Mesg.iData == gts)
                                newChar = lts;
                        }
                        StyleLetters(true, *(new FaChar(newChar)));
                    #else
                        StyleLetters(true, *(new FaChar(Mesg.iData)));
                    #endif
                    mMarkAnchor = mMarkEnd;
            }
            break;
        case PM_KEY_RELEASE:
            CPEditBox::Message(Mesg);
            if (Mesg.iData == PK_SHIFT) shift = false;
            break;
        
		default:
	        return CPEditBox::Message(Mesg);
    }
    return 0;
}

#ifdef WIN32
/** Sends a PM_KEY message to 'this' with Mesg.iData = btn. */
void FaTextBox::SendKeyMsg(WORD btn)
{
    PegMessage keyMsg(this, PM_KEY);
    keyMsg.iData = btn;
    MessageQueue()->Push(keyMsg);
}

/** Inserts a char at cursor and moves the cursor to the appropriate place. */
void FaTextBox::InsertCharEx(WORD chr)
{
    static WORD prevChr = 0;
    static WORD bprevChr = 0;
    Farsi *farsi = new Farsi;
    switch (prevChr)
    {
        case PK_CR:
            HomeCursor();
            SendKeyMsg(PK_CR);
        case space:
            if (farsi->Type( *(new RawFaChar(chr)) ) == RTL) HomeCursor();
            if (farsi->Type( *(new RawFaChar(bprevChr)) ) == RTL)
            {
                SendKeyMsg(space);
            }
            else if (farsi->Type( *(new RawFaChar(bprevChr)) ) == LTR)
            {
                InsertCharAtCursor(space);
            }
            else
            {
                InsertCharAtCursor(space);
                MoveCursor(-1);
            }
            if (farsi->Type( *(new RawFaChar(chr)) ) == RTL) HomeCursor();
            break;
        default:
            if (farsi->Type( *(new RawFaChar(prevChr)) ) == RTL)
            {
                if (farsi->Type( *(new RawFaChar(bprevChr)) ) == LTR)
                {
                    HomeCursor();
                }
            }
            SendKeyMsg(prevChr);
    }
    bprevChr = prevChr;
    prevChr = chr;
    MessageQueue()->Flush();
}

/** Converts a normal char to a pk_ char. */
WORD FaTextBox::ChrToPChr(WORD chr)
{
    WORD pk_chr;
    switch (chr)
    {
        case vspace:              pk_chr = pk_vspace; break;
        case oparen:              pk_chr = olparen;   break;
        case cparen:              pk_chr = clparen;   break;
        case qmark:               pk_chr = pk_qmark;  break;
        case scolon:              pk_chr = pk_scolon; break;
        case comma:               pk_chr = pk_comma;  break;
        case oquote:              pk_chr = pk_oquote; break;
        case cquote:              pk_chr = pk_cquote; break;
        case tatweel:             pk_chr = pk_tatweel;break;
        case digit0:              pk_chr = ldigit0;   break;
        case digit1:              pk_chr = ldigit1;   break;
        case digit2:              pk_chr = ldigit2;   break;
        case digit3:              pk_chr = ldigit3;   break;
        case digit4:              pk_chr = ldigit4;   break;
        case digit5:              pk_chr = ldigit5;   break;
        case digit6:              pk_chr = ldigit6;   break;
        case digit7:              pk_chr = ldigit7;   break;
        case digit8:              pk_chr = ldigit8;   break;
        case digit9:              pk_chr = ldigit9;   break;
        case aa:     case _aa:    pk_chr = pk_aa;     break;
        case alef:   case _alef:  pk_chr = pk_alef;   break;
        case beh_:   case _beh_:
        case _beh:   case beh:    pk_chr = pk_beh;    break;
        case peh_:   case _peh_:
        case _peh:   case peh:    pk_chr = pk_peh;    break;
        case teh_:   case _teh_:
        case _teh:   case teh:    pk_chr = pk_teh;    break;
        case ceh_:   case _ceh_:
        case _ceh:   case ceh:    pk_chr = pk_ceh;    break;
        case jim_:   case _jim_:
        case _jim:   case jim:    pk_chr = pk_jim;    break;
        case cheh_:  case _cheh_:
        case _cheh:  case cheh:   pk_chr = pk_cheh;   break;
        case ha_:    case _ha_:
        case _ha:    case ha:     pk_chr = pk_ha;     break;
        case kheh_:  case _kheh_:
        case _kheh:  case kheh:   pk_chr = pk_kheh;   break;
        case dal:    case _dal:   pk_chr = pk_dal;    break;
        case zal:    case _zal:   pk_chr = pk_zal;    break;
        case reh:    case _reh:   pk_chr = pk_reh;    break;
        case zeh:    case _zeh:   pk_chr = pk_zeh;    break;
        case jeh:    case _jeh:   pk_chr = pk_jeh;    break;
        case sin_:   case _sin_:
        case _sin:   case __sin:  pk_chr = pk_sin;    break;
        case shin_:  case _shin_:
        case _shin:  case shin:   pk_chr = pk_shin;   break;
        case sad_:   case _sad_:
        case _sad:   case sad:    pk_chr = pk_sad;    break;
        case zad_:   case _zad_:
        case _zad:   case zad:    pk_chr = pk_zad;    break;
        case ta_:    case _ta_:
        case _ta:    case ta:     pk_chr = pk_ta;     break;
        case za_:    case _za_:
        case _za:    case za:     pk_chr = pk_za;     break;
        case ein_:   case _ein_:
        case _ein:   case ein:    pk_chr = pk_ein;    break;
        case ghein_: case _ghein_:
        case _ghein: case ghein:  pk_chr = pk_ghein;  break;
        case feh_:   case _feh_:
        case _feh:   case feh:    pk_chr = pk_feh;    break;
        case ghaf_:  case _ghaf_:
        case _ghaf:  case ghaf:   pk_chr = pk_ghaf;   break;
        case kaf_:   case _kaf_:
        case _kaf:   case kaf:    pk_chr = pk_kaf;    break;
        case gaf_:   case _gaf_:
        case _gaf:   case gaf:    pk_chr = pk_gaf;    break;
        case lam_:   case _lam_:
        case _lam:   case lam:    pk_chr = pk_lam;    break;
        case mim_:   case _mim_:
        case _mim:   case mim:    pk_chr = pk_mim;    break;
        case noon_:  case _noon_:
        case _noon:  case noon:   pk_chr = pk_noon;   break;
        case vav:    case _vav:   pk_chr = pk_vav;    break;
        case heh_:   case _heh_:
        case _heh:   case heh:    pk_chr = pk_heh;    break;
        case yeh_:   case _yeh_:
        case _yeh:   case yeh:    pk_chr = pk_yeh;    break;
        case talef:  case _talef: pk_chr = pk_alef*0x100 + pk_tfath; break;
        case halef:  case _halef: pk_chr = pk_halef;  break;
        case alefh:  case _alefh: pk_chr = pk_alefh;  break;
        case hvav:   case _hvav:  pk_chr = pk_hvav;   break;
        case hyeh_:  case _hyeh_:
        case _hyeh:  case hyeh:   pk_chr = pk_hyeh;   break;
        case hamza:               pk_chr = pk_hamza;  break;
        case hheh:   case _hheh:  pk_chr = pk_dheh;   break;
        case dheh:   case _dheh:  pk_chr = pk_dheh;   break;
        case akaf_:  case _akaf_:
        case _akaf:  case akaf:   pk_chr = pk_akaf;   break;
        case yehd_:  case _yehd_:
        case _yehd:  case yehd:   pk_chr = pk_yehd;   break;
        case vconn:               pk_chr = pk_vconn;  break;
        default:                  pk_chr = lqmark;    break;
    }
    return pk_chr;
}
#endif

/** This function can insert a char into the text and set the correct style of
the letters (connections), then set the cursor's correct position. If you leave
the 'Curr' argument empty, the function just corrects the styles. */
void FaTextBox::StyleLetters(bool doRedraw, FaChar Curr)
{
    if (Curr.FullChar() == 0x00)
        if (GetCharAtCursor() == NULL)
        {
            Redraw();
            return;
        }
    
    FaChar Next;
    FaChar Prev;
    Farsi *farsi = new Farsi();
    
    if (GetCharAtCursor() != NULL)
        Next.Set((CP_CHAR)GetCharAtCursor()[0], (CP_CHAR)GetCharAtCursor()[1]);
    if (mCursor.x != 0)
    {
        MoveCursor(-1);
        Prev.Set((CP_CHAR)GetCharAtCursor()[0], (CP_CHAR)GetCharAtCursor()[1]);
        RemoveCharAtCursor(false); //Removes Prev
    }
    
    if (Curr.FullChar() == 0)
    {
        if (farsi->CanConnectToNext(Prev))
        {
            if (farsi->CanConnectToPrev(Next))
            {
                farsi->ConnectToPrev(&Next);
                farsi->ConnectToNext(&Prev);
            }
            else
            {
                farsi->DisconnectFromPrev(&Next);
                farsi->DisconnectFromNext(&Prev);
            }
        }
        else
        {
            farsi->DisconnectFromPrev(&Next);
            farsi->DisconnectFromNext(&Prev);
        }
    }
    else
        farsi->SetCorrectStyles(&Prev, &Curr, &Next);
    
    if (Prev.FullChar() != 0) InsertCharAtCursor(Prev.FullChar());
    if (Curr.FullChar() != 0) InsertCharAtCursor(Curr.FullChar());
    if ( !((Next.FullChar() == 0) || (Next.FullChar() == CP_CHAR_CR)) )
    {
        RemoveCharAtCursor(false); //Removes Next
        InsertCharAtCursor(Next.FullChar());
        MoveCursor(-1);
    }
    
    if (farsi->Type(Curr) == RTL)
    {
        MoveCursor(-1);
        if (doRedraw)
            Redraw();
        return;
    }
    else if (farsi->Type(Curr) == SPACE)
    {
        if (farsi->Type(Next) == NIL)
        {
            if (farsi->Type(Prev) == NIL)
            {
                MoveCursor(-1);
            }
            else if (farsi->Type(Prev) == LTR)
            {
                RemoveCharAtCursor(true);
                int type = LTR;
                FaChar fchr;
                while(type == LTR)
                {
                    MoveCursor(-1);
                    fchr.Set((CP_CHAR)GetCharAtCursor()[0], (CP_CHAR)GetCharAtCursor()[1]);
                    type = farsi->Type(fchr);
                    if (mCursor.x == 0)
                        break;
                }
                if (type != LTR)
                    MoveCursor(1);
                InsertCharAtCursor(space);
                MoveCursor(-1);
            }
        }
        else if (farsi->Type(Next) == RTL)
        {
            MoveCursor(-1);
        }
        else if (farsi->Type(Next) == LTR)
        {
            if (farsi->Type(Prev) != LTR)
            {
                MoveCursor(-1);
            }
        }
        else if (farsi->Type(Next) == SPACE)
        {
            if (farsi->Type(Prev) == LTR)
            {
                RemoveCharAtCursor(true);
                int type = LTR;
                FaChar fchr;
                while ((type == LTR))
                {
                    MoveCursor(-1);
                    fchr.Set((CP_CHAR)GetCharAtCursor()[0], (CP_CHAR)GetCharAtCursor()[1]);
                    type = farsi->Type(fchr);
                    if (mCursor.x == 0)
                        break;
                }
                if (type != LTR)
                    MoveCursor(1);
                InsertCharAtCursor(space);
            }
            MoveCursor(-1);
        }
    }
    
    if (doRedraw)
        Redraw();
}

/** Performs a backspace. */
void FaTextBox::Backspace()
{
    if (GetCharAtCursor() == NULL)
        return;
    
    // Variables ///////////////////////////////////////////////////////////
    FaChar Prev;
    FaChar Next;
    Farsi *farsi = new Farsi();
    
    // Set Variables ///////////////////////////////////////////////////////
    Next.Set((CP_CHAR)GetCharAtCursor()[0], (CP_CHAR)GetCharAtCursor()[1]);
    if (mCursor.x != 0)
    {
        MoveCursor(-1);
        Prev.Set((CP_CHAR)GetCharAtCursor()[0], (CP_CHAR)GetCharAtCursor()[1]);
        MoveCursor(1);
    }
    
    // Remove the corresponding char ///////////////////////////////////////
    if (farsi->Type(Prev) == NIL)
    {
        if (farsi->Type(Next) == NIL)
        {
            if (mCursor.y != 0)
            {
                mCursor.y = mCursor.y - 1;
                mCursor.x = 0;
                while (GetCharAtCursor()[0] != (PEGCHAR)CP_CHAR_CR)
                {
                    MoveCursor(1);
                }
                RemoveCharAtCursor(false);
                mCursor.x = 0;
            }
        }
        else if (farsi->Type(Next) == RTL)
        {
            RemoveCharAtCursor(false);
        }
        else if (farsi->Type(Next) == LTR)
        {
            if (mCursor.y != 0)
                RemoveCharAtCursor(true);
        }
        else if (farsi->Type(Next) == SPACE)
        {
            RemoveCharAtCursor(false);
        }
    }
    else if (farsi->Type(Prev) == RTL)
    {
        if (farsi->Type(Next) == NIL)
        {
            if (mCursor.y != 0)
            {
                FaChar fChr;
                CPString Str("");
                while (mCursor.x != 0)
                {
                    MoveCursor(-1);
                    fChr.Set((CP_CHAR)GetCharAtCursor()[0], (CP_CHAR)GetCharAtCursor()[1]);
                    Str += fChr.FullChar();
                    RemoveCharAtCursor(false);
                }
                fChr.Set((CP_CHAR)GetCharAtCursor()[0], (CP_CHAR)GetCharAtCursor()[1]);
                
                mCursor.y = mCursor.y - 1;
                mCursor.x = 0;
                while (GetCharAtCursor()[0] != (PEGCHAR)CP_CHAR_CR)
                {
                    MoveCursor(1);
                }
                RemoveCharAtCursor(false);
                
                mCursor.x = 0;
                int l = Str.Length();
                for (int i = 0; i < l; i++)
                    InsertCharAtCursor(Str[l - i - 1]);
            }
        }
        else if (farsi->Type(Next) == LTR)
        {
            RemoveCharAtCursor(true);
        }
        else
        {
            RemoveCharAtCursor(false);
        }
    }
    else if (farsi->Type(Prev) == LTR)
    {
        if (farsi->Type(Next) == RTL)
        {
            RemoveCharAtCursor(false);
        }
        else
        {
            RemoveCharAtCursor(true);
        }
    }
    else if (farsi->Type(Prev) == SPACE)
    {
        if (farsi->Type(Next) == NIL)
        {
            if (GetCharAtCursor()[0] == (PEGCHAR)CP_CHAR_CR)
                RemoveCharAtCursor(false);
        }
        else if (farsi->Type(Next) == LTR)
        {
            RemoveCharAtCursor(true);
        }
        else
        {
            RemoveCharAtCursor(false);
        }
    }
    StyleLetters();
}

/** Inserts a new line into the text. */
void FaTextBox::NewLine()
{
    FaChar fChr;
    Farsi *farsi = new Farsi();
    if (GetCharAtCursor() == NULL)
    {
        InsertCharAtCursor(CP_CHAR_CR);
        return;
    }
    if ((CP_CHAR)GetCharAtCursor()[0] == 0)
    {
        HomeCursor();
        InsertCharAtCursor(CP_CHAR_CR);
        fChr.Set((CP_CHAR)GetCharAtCursor()[0], (CP_CHAR)GetCharAtCursor()[1]);
        while ( !((fChr.FullChar() == CP_CHAR_CR) || (fChr.FullChar() == CP_EOS)) )
        {
            MoveCursor(1);
            fChr.Set((CP_CHAR)GetCharAtCursor()[0], (CP_CHAR)GetCharAtCursor()[1]);
        }
        return;
    }
    if (GetCharAtCursor()[0] == (PEGCHAR)CP_CHAR_CR)
    {
        if (mCursor.x == 0)
            InsertCharAtCursor(CP_CHAR_CR);
        else
        {
            SIGNED cx = mCursor.x;
            mCursor.x = 0;
            InsertCharAtCursor(CP_CHAR_CR);
            mCursor.x = cx;
        }
        return;
    }
    CPString Str("");
    while (mCursor.x != 0)
    {
        MoveCursor(-1);
        fChr.Set((CP_CHAR)GetCharAtCursor()[0], (CP_CHAR)GetCharAtCursor()[1]);
        Str += fChr.FullChar();
        RemoveCharAtCursor(false);
    }
    fChr.Set((CP_CHAR)GetCharAtCursor()[0], (CP_CHAR)GetCharAtCursor()[1]);
    if (farsi->Type(fChr) == RTL)
    {
        farsi->DisconnectFromPrev(&fChr);
        RemoveCharAtCursor(false);
        InsertCharAtCursor(fChr.FullChar());
        fChr.Set((CP_CHAR)GetCharAtCursor()[0], (CP_CHAR)GetCharAtCursor()[1]);
    }
    while ( !((fChr.FullChar() == CP_CHAR_CR) || (fChr.FullChar() == CP_EOS)) )
    {
        MoveCursor(1);
        fChr.Set((CP_CHAR)GetCharAtCursor()[0], (CP_CHAR)GetCharAtCursor()[1]);
    }
    InsertCharAtCursor(CP_CHAR_CR);
    int l = Str.Length();
    for (int i = 0; i < l; i++)
        InsertCharAtCursor(Str[l - i - 1]);
    MoveCursor(-1);
    fChr.Set((CP_CHAR)GetCharAtCursor()[0], (CP_CHAR)GetCharAtCursor()[1]);
    if (farsi->Type(fChr) == RTL)
    {
        farsi->DisconnectFromNext(&fChr);
        RemoveCharAtCursor(false);
        InsertCharAtCursor(fChr.FullChar());
    }
    else
        MoveCursor(1);
}

