#include "Farsi.h"

/** Checks whether a Farsi char can connect to its previous char or not. */
bool Farsi::CanConnectToPrev(FaChar fchr)
{
    switch (NumberOfStyles(fchr))
    {
        case 2:
            return false;
        case 4:
            switch (Style(fchr))
            {
                case initial: case medial: case final: case isolated:
                    return true;
                default:
                    return false;
            }
        default:
            return false;
    }
    return false;
}

/** Checks whether a Farsi char can connect to its next char or not. */
bool Farsi::CanConnectToNext(FaChar fchr)
{
    switch (Style(fchr))
    {
        case initial: case medial: case final: case isolated:
            return true;
        default:
            return false;
    }
    return false;
}

/** Changes the style of 'fChr' so that it is connected to its previous char. */
void Farsi::ConnectToPrev(FaChar *fchr)
{
    switch (Style(*fchr))
    {
        case final:
            SetStyle(fchr, medial);
            break;
        case isolated:
            SetStyle(fchr, initial);
            break;
    }
}

/** Changes the style of 'fChr' so that it is connected to its next char. */
void Farsi::ConnectToNext(FaChar *fchr)
{
    switch (Style(*fchr))
    {
        case initial:
            SetStyle(fchr, medial);
            break;
        case isolated:
            SetStyle(fchr, final);
            break;
    }
}

/** Changes the style of 'fChr' so that it is disconnected from its previous char. */
void Farsi::DisconnectFromPrev(FaChar *fchr)
{
    switch (Style(*fchr))
    {
        case initial:
            SetStyle(fchr, isolated);
            break;
        case medial:
            SetStyle(fchr, final);
            break;
    }
}

/** Changes the style of 'fChr' so that it is disconnected from its next char. */
void Farsi::DisconnectFromNext(FaChar *fchr)
{
    switch (Style(*fchr))
    {
        case medial:
            SetStyle(fchr, initial);
            break;
        case final:
            SetStyle(fchr, isolated);
    }
}

/** Returns the 'Type' of 'fChr'. Type can be NIL, RTL, LTR, SPACE. Note that
'NIL' means a NULL character or a CR character. */
int Farsi::Type(FaChar fchr)
{
    if (fchr.c1 == 0xEC)
    {
        switch (Style(fchr))
        {
            case _sign: case initial: case medial: case final: case isolated:
                return RTL;
        }
    }
    else
    {
        switch(fchr.c1)
        {
            #ifdef WIN32
            case pk_vspace:case pk_vconn:case pk_comma:case pk_oquote:case pk_scolon:
            case pk_cquote:case pk_qmark:case pk_hamza:case pk_peh:   case pk_cheh:
            case pk_jeh:   case pk_gaf:  case pk_kaf:  case pk_aa:    case pk_halef:
            case pk_hvav:  case pk_alefh:case pk_hyeh: case pk_alef:  case pk_beh:
            case pk_dheh:  case pk_teh:  case pk_ceh:  case pk_jim:   case pk_ha:
            case pk_kheh:  case pk_dal:  case pk_zal:  case pk_reh:   case pk_zeh:
            case pk_sin:   case pk_shin: case pk_sad:  case pk_zad:   case pk_multis:
            case pk_ta:    case pk_za:   case pk_ein:  case pk_ghein: case pk_tatweel:
            case pk_feh:   case pk_ghaf: case pk_akaf: case pk_lam:   case pk_mim:
            case pk_noon:  case pk_heh:  case pk_vav:  case pk_yeh:   case pk_yehd:
            case pk_divs:
            #endif
            case excal: case dot: case colon: case csbrac: case osbrac: case ccbrac: case ocbrac:
                return RTL;
            case space:
                return SPACE;
            case CP_EOS: case CP_CHAR_CR:
                return NIL;
        }
    }
    return LTR;
}

/** Returns the 'Style' of 'fChr'. Style can be _sign, number, initial, medial,
final, isolated, ind. Note that 'ind' means that the style isn't defined, e.g
if you give an english char to this function, it returns 'ind'. */
int Farsi::Style(FaChar fchr)
{
    switch(fchr.FullChar())
    {
        #ifdef WIN32
        case pk_vspace:case pk_vconn:case pk_comma:case pk_oquote:case pk_scolon:
        case pk_cquote:case pk_qmark:case pk_hamza:case pk_peh:   case pk_cheh:
        case pk_jeh:   case pk_gaf:  case pk_kaf:  case pk_aa:    case pk_halef:
        case pk_hvav:  case pk_alefh:case pk_hyeh: case pk_alef:  case pk_beh:
        case pk_dheh:  case pk_teh:  case pk_ceh:  case pk_jim:   case pk_ha:
        case pk_kheh:  case pk_dal:  case pk_zal:  case pk_reh:   case pk_zeh:
        case pk_sin:   case pk_shin: case pk_sad:  case pk_zad:   case pk_multis:
        case pk_ta:    case pk_za:   case pk_ein:  case pk_ghein: case pk_tatweel:
        case pk_feh:   case pk_ghaf: case pk_akaf: case pk_lam:   case pk_mim:
        case pk_noon:  case pk_heh:  case pk_vav:  case pk_yeh:   case pk_yehd:
        case pk_divs:
        #endif
        case vspace:case oparen:case cparen: case qmark: case scolon:
        case comma: case oquote:case cquote: case hamza:
            return _sign;
        case digit0:case digit1:case digit2:case digit3: case digit4:
        case digit5:case digit6:case digit7:case digit8: case digit9:
            return number;
        case beh_:  case peh_:  case teh_:  case ceh_:   case jim_:  case cheh_:
        case ha_:   case kheh_: case sin_:  case shin_:  case sad_:  case zad_:
        case ta_:   case za_:   case ein_:  case ghein_: case feh_:  case ghaf_:
        case kaf_:  case gaf_:  case lam_:  case mim_:   case noon_: case heh_:
        case yeh_:  case hyeh_: case akaf_: case yehd_:
            return initial;
        case _beh_: case _peh_: case _teh_: case _ceh_:  case _jim_: case _cheh_:
        case _ha_:  case _kheh_:case _sin_: case _shin_: case _sad_: case _zad_:
        case _ta_:  case _za_:  case _ein_: case _ghein_:case _feh_: case _ghaf_:
        case _kaf_: case _gaf_: case _lam_: case _mim_:  case _noon_:case _heh_:
        case _yeh_: case _hyeh_:case _akaf_:case _yehd_: case vconn: case tatweel:
            return medial;
        case _aa:   case _alef: case _beh:  case _peh:   case _teh:  case _ceh:
        case _jim:  case _cheh: case _ha:   case _kheh:  case _dal:  case _zal:
        case _reh:  case _zeh:  case _jeh:  case _sin:   case _shin: case _sad:
        case _zad:  case _ta:   case _za:   case _ein:   case _ghein:case _feh:
        case _ghaf: case _kaf:  case _gaf:  case _lam:   case _mim:  case _noon:
        case _vav:  case _heh:  case _yeh:  case _talef: case _halef:case _alefh:
        case _hvav: case _hyeh: case _hheh: case _dheh:  case _akaf: case _yehd:
            return final;
        case aa:    case alef:  case beh:   case peh:    case teh:   case ceh:
        case jim:   case cheh:  case ha:    case kheh:   case dal:   case zal:
        case reh:   case zeh:   case jeh:   case __sin:  case shin:  case sad:
        case zad:   case ta:    case za:    case ein:    case ghein: case feh:
        case ghaf:  case kaf:   case gaf:   case lam:    case mim:   case noon:
        case vav:   case heh:   case yeh:   case talef:  case halef: case alefh:
        case hvav:  case hyeh:  case hheh:  case dheh:   case akaf:  case yehd:
            return isolated;
        default:
            return ind;
    }
    return 0;
}

/** Returns number of the styles of a char. */
int Farsi::NumberOfStyles(FaChar fchr)
{
    switch (fchr.FullChar())
    {
        case aa:    case alef:  case dal:   case zal:    case reh:   case zeh:
        case jeh:   case vav:   case talef: case halef: case alefh:  case hvav:
        case hheh:  case dheh:
        case _aa:   case _alef: case _dal:  case _zal:   case _reh:  case _zeh:
        case _jeh:  case _vav:  case _talef:case _halef:case _alefh: case _hvav:
        case _hheh: case _dheh:
            return 2;
        case beh_:  case peh_:  case teh_:  case ceh_:   case jim_:  case cheh_:
        case ha_:   case kheh_: case sin_:  case shin_:  case sad_:  case zad_:
        case ta_:   case za_:   case ein_:  case ghein_: case feh_:  case ghaf_:
        case kaf_:  case gaf_:  case lam_:  case mim_:   case noon_: case heh_:
        case yeh_:  case hyeh_: case akaf_: case yehd_:
        case _beh_: case _peh_: case _teh_: case _ceh_:  case _jim_: case _cheh_:
        case _ha_:  case _kheh_:case _sin_: case _shin_: case _sad_: case _zad_:
        case _ta_:  case _za_:  case _ein_: case _ghein_:case _feh_: case _ghaf_:
        case _kaf_: case _gaf_: case _lam_: case _mim_:  case _noon_:case _heh_:
        case _yeh_: case _hyeh_:case _akaf_:case _yehd_: case vconn: case tatweel:
        case _beh:  case _peh:  case _teh:  case _ceh:   case _jim:  case _cheh:
        case _ha:   case _kheh: case _sin:  case _shin:  case _sad:  case _zad:
        case _ta:   case _za:   case _ein:  case _ghein: case _feh:  case _ghaf:
        case _kaf:  case _gaf:  case _lam:  case _mim:   case _noon: case _heh:
        case _yeh:  case _hyeh: case _akaf: case _yehd:
        case beh:   case peh:   case teh:   case ceh:    case jim:   case cheh:
        case ha:    case kheh:  case __sin: case shin:   case sad:   case zad:
        case ta:    case za:    case ein:   case ghein:  case feh:   case ghaf:
        case kaf:   case gaf:   case lam:   case mim:    case noon:  case heh:
        case yeh:   case hyeh:  case akaf:  case yehd:
            return 4;
        default:
            return 1;
    }
    return 1;
}

/** Sets the style of 'fChr' to 'toStyle' that can be 'initial', 'medial', etc. */
void Farsi::SetStyle(FaChar *fchr, int toStyle)
{
    switch (NumberOfStyles(*fchr))
    {
        // Two-style letters:
        case 2:
            switch (Style(*fchr))
            {
                // If two-style char is final:
                case final:
                    switch (toStyle)
                    {
                        // Convert final to:
                        case isolated:
                            fchr->c2 -= 1;
                        default:
                            return;
                    }
                
                // If two-style char is isolated:
                case isolated:
                    switch (toStyle)
                    {
                        // Convert isolated to:
                        case final:
                            fchr->c2 += 1;
                        default:
                            return;
                    }
                default:
                    return;
            }
            
        // Four-style letters:
        case 4:
            if ((fchr->FullChar() == vconn) || (fchr->FullChar() == tatweel))
            {
                fchr->c2 = fchr->FullChar() - 0xEC00;
                return;
            }
            switch (Style(*fchr))
            {
                // If four-style char is inital:
                case initial:
                    switch (toStyle)
                    {
                        // Convert initial to:
                        case medial:
                            fchr->c2 += 1;
                            return;
                        case final:
                            fchr->c2 += 2;
                            return;
                        case isolated:
                            fchr->c2 += 3;
                        default:
                            return;
                    }
                    
                // If four-style char is medial:
                case medial:
                    switch (toStyle)
                    {
                        // Convert medial to:
                        case initial:
                            fchr->c2 -= 1;
                            return;
                        case final:
                            fchr->c2 += 1;
                            return;
                        case isolated:
                            fchr->c2 += 2;
                        default:
                            return;
                    }
                    
                // If four-style char is final:
                case final:
                    switch (toStyle)
                    {
                        // Convert final to:
                        case initial:
                            fchr->c2 -= 2;
                            return;
                        case medial:
                            fchr->c2 -= 1;
                            return;
                        case isolated:
                            fchr->c2 += 1;
                        default:
                            return;
                    }
                    
                // If four-style char is isolated:
                case isolated:
                    switch (toStyle)
                    {
                        // Convert isolated to:
                        case initial:
                            fchr->c2 -= 3;
                            return;
                        case medial:
                            fchr->c2 -= 2;
                            return;
                        case final:
                            fchr->c2 -= 1;
                        default:
                            return;
                    }
                    
                default:
                    return;
            }
    }
}

/** Sets the correct connections between Prev-Curr and Curr-Next. */
void Farsi::SetCorrectStyles(FaChar *Prev, FaChar *Curr, FaChar *Next)
{
    if (CanConnectToNext(*Prev))
    {
        if (CanConnectToPrev(*Curr))
        {
            ConnectToPrev(Curr);
            ConnectToNext(Prev);
        }
        else
        {
            DisconnectFromPrev(Curr);
            DisconnectFromNext(Prev);
        }
    }
    else
    {
        DisconnectFromPrev(Curr);
        DisconnectFromNext(Prev);
    }
    
    if (CanConnectToNext(*Curr))
    {
        if (CanConnectToPrev(*Next))
        {
            ConnectToPrev(Next);
            ConnectToNext(Curr);
        }
        else
        {
            DisconnectFromPrev(Next);
            DisconnectFromNext(Curr);
        }
    }
    else
    {
        DisconnectFromPrev(Next);
        DisconnectFromNext(Curr);
    }
}

