#ifndef FARSI_INCLUDED
#define FARSI_INCLUDED

#include "ClassPad.h"
#include "FaChar.cpp"

class Farsi
{
public:
    //Connection
    bool CanConnectToPrev(FaChar fchr);
    bool CanConnectToNext(FaChar fchr);
    void ConnectToPrev(FaChar *fchr);
    void ConnectToNext(FaChar *fchr);
    void DisconnectFromPrev(FaChar *fchr);
    void DisconnectFromNext(FaChar *fchr);
    
    //Types
    int Type(FaChar fchr);
    
    //Styles
    int Style(FaChar fchr);
    int NumberOfStyles(FaChar fchr);
    void SetStyle(FaChar *fchr, int toStyle);
    void SetCorrectStyles(FaChar *Prev, FaChar *Curr, FaChar *Next);
};

enum CharType
{
    NIL,
    RTL,
    LTR,
    SPACE,
};

enum CharStyle
{
    _sign,
    number,
    initial,
    medial,
    final,
    isolated,
    ind,
};

enum Latin_Farsi_Chars
{
    space   = 0x20,
    excal   = 0x21,
    pcs     = 0x25,
    aps     = 0x27,
    clparen = 0x28,
    olparen = 0x29,
    star    = 0x2A,
    plus    = 0x2B,
    lcomma  = 0x2C,
    minus   = 0x2D,
    dot     = 0x2E,
    slash   = 0x2F,
    ldigit0 = 0x30,
    ldigit1 = 0x31,
    ldigit2 = 0x32,
    ldigit3 = 0x33,
    ldigit4 = 0x34,
    ldigit5 = 0x35,
    ldigit6 = 0x36,
    ldigit7 = 0x37,
    ldigit8 = 0x38,
    ldigit9 = 0x39,
    colon   = 0x3A,
    lts     = 0x3C,
    equals  = 0x3D,
    gts     = 0x3E,
    lqmark  = 0x3F,
    csbrac  = 0x5B,
    osbrac  = 0x5D,
    caret   = 0x5E,
    ccbrac  = 0x7B,
    ocbrac  = 0x7D,
    vari    = 0xEDA9,
    varj    = 0xEDAA,
    vark    = 0xEDAB,
    varx    = 0xEDB8,
    vary    = 0xEDB9,
    varz    = 0xEDBA,
    eulers  = 0xEE02,
    pins    = 0xEE04,
    infs    = 0xEE05,
    multis  = 0xEE10,
    divs    = 0xEE15,
    ahome   = 0xEE63,
    aend    = 0xEE65,
    ardel   = 0xEE86,
    aldel   = 0xEE88,
    barrow  = 0xEEA1,
};

enum Farsi_Chars
{
    vspace = 0xEC01, //Virtual Space
    oparen,
    cparen,
    qmark,
    scolon,
    comma,
    oquote,
    cquote,
    //0xEC09 is a special character
    tatweel = 0xEC0A,
    digit0,
    digit1,
    digit2,
    digit3,
    digit4,
    digit5,
    digit6,
    digit7,
    digit8,
    digit9,
    aa,
    _aa,
    alef,
    _alef,
    beh_,
    _beh_,
    _beh,
    beh,
    peh_,
    _peh_,
    _peh,
    peh,
    teh_,
    _teh_,
    _teh,
    teh,
    ceh_,
    _ceh_,
    _ceh,
    ceh,
    jim_,
    _jim_,
    _jim,
    jim,
    cheh_,
    _cheh_,
    _cheh,
    cheh,
    ha_,
    _ha_,
    _ha,
    ha,
    kheh_,
    _kheh_,
    _kheh,
    kheh,
    dal,
    _dal,
    zal,
    _zal,
    reh,
    _reh,
    zeh,
    _zeh,
    jeh,
    _jeh,
    sin_,
    _sin_,
    _sin,
    __sin, //Because 'sin' is defined as a math function, we use '__sin' instead.
    shin_,
    _shin_,
    _shin,
    shin,
    sad_,
    _sad_,
    _sad,
    sad,
    zad_,
    _zad_,
    _zad,
    zad,
    ta_,
    _ta_,
    _ta,
    ta,
    za_,
    _za_,
    _za,
    za,
    ein_,
    _ein_,
    _ein,
    ein,
    ghein_,
    _ghein_,
    _ghein,
    ghein,
    feh_,
    _feh_,
    _feh,
    feh,
    //0xEC67 ~ 0xEC7E are Greek letters
    ghaf_ = 0xEC7F,
    _ghaf_,
    _ghaf,
    ghaf,
    kaf_,
    _kaf_,
    _kaf,
    kaf,
    gaf_,
    _gaf_,
    _gaf,
    gaf,
    lam_,
    _lam_,
    _lam,
    lam,
    mim_,
    _mim_,
    _mim,
    mim,
    noon_,
    _noon_,
    _noon,
    noon,
    vav,
    _vav,
    heh_,
    _heh_,
    _heh,
    heh,
    yeh_,
    _yeh_,
    _yeh,
    yeh,
    //0xECA1 ~ 0xECBA are variable letters
    talef = 0xECBB,
    _talef,
    halef,
    _halef,
    alefh,
    _alefh,
    hvav,
    _hvav,
    hyeh_,
    _hyeh_,
    _hyeh,
    hyeh,
    hamza,
    hheh,
    _hheh,
    dheh,
    _dheh,
    akaf_,
    _akaf_,
    _akaf,
    akaf,
    yehd_,
    _yehd_,
    _yehd,
    yehd,
    vconn, //Virtual Connection
};

#ifdef WIN32
enum PC_Farsi_Keys
{
    pk_line    = 0x0A,
    pk_peh     = 0x81,
    pk_cheh    = 0x8D,
    pk_jeh,
    pk_gaf     = 0x90,
    pk_kaf     = 0x98,
    pk_vspace  = 0x9D,
    pk_vconn,
    pk_comma   = 0xA1,
    pk_oquote  = 0xAB,
    pk_scolon  = 0xBA,
    pk_cquote,
    pk_qmark   = 0xBF,
    pk_hamza   = 0xC1,
    pk_aa,
    pk_halef,
    pk_hvav,
    pk_alefh,
    pk_hyeh,
    pk_alef,
    pk_beh,
    pk_dheh,
    pk_teh,
    pk_ceh,
    pk_jim,
    pk_ha,
    pk_kheh,
    pk_dal,
    pk_zal,
    pk_reh,
    pk_zeh,
    pk_sin,
    pk_shin,
    pk_sad,
    pk_zad,
    pk_multis,
    pk_ta,
    pk_za,
    pk_ein,
    pk_ghein,
    pk_tatweel,
    pk_feh,
    pk_ghaf,
    pk_akaf,
    pk_lam     = 0xE1,
    pk_mim     = 0xE3,
    pk_noon,
    pk_heh,
    pk_vav,
    pk_yeh     = 0xEC,
    pk_yehd,
    pk_tfath   = 0xF0,
    pk_tzam,
    pk_tjar,
    pk_fath,
    pk_zam     = 0xF5,
    pk_jar,
    pk_divs,
    pk_tashdid,
    pk_sokun   = 0xFA,
};
#endif

#endif

